@@ -1,7 +1,7 @@
-/* $Id: macros.h,v 1.5 2019/09/24 09:37:52 nanard Exp $ */
+/* $Id: macros.h,v 1.6 2021/08/21 08:26:48 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2012-2019 Thomas Bernard
+ * (c) 2012-2021 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -16,6 +16,11 @@
 #define FALL_THROUGH
 #endif
 
+/* Macro to print errors during initialization.
+ * Print them on both stderr and syslog.
+ * if debug_flag is on, syslog already print on console */
+#define INIT_PRINT_ERR(...) do { if (!debug_flag) fprintf(stderr, __VA_ARGS__); syslog(LOG_ERR, __VA_ARGS__); } while(0)
+
 #include <stdint.h>
 
 #ifndef INLINE
