@@ -1,9 +1,9 @@
-/* $Id: options.c,v 1.39 2020/04/09 18:40:42 nanard Exp $ */
+/* $Id: options.c,v 1.40 2021/08/21 08:22:46 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * MiniUPnP project
  * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
  * author: Ryan Wagoner
- * (c) 2006-2020 Thomas Bernard
+ * (c) 2006-2021 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -19,6 +19,7 @@
 #include "pcplearndscp.h"
 #endif /* PCP_SADSPC */
 #include "upnpglobalvars.h"
+#include "macros.h"
 
 #ifndef DISABLE_CONFIG_FILE
 struct option * ary_options = NULL;
@@ -90,6 +91,9 @@ static const struct {
 #endif
 #ifdef ENABLE_LEASEFILE
 	{ UPNPLEASEFILE, "lease_file"},
+#ifdef ENABLE_UPNPPINHOLE
+	{ UPNPLEASEFILE6, "lease_file6"},
+#endif
 #endif
 #ifdef IGD_V2
 	{ UPNPFORCEIGDDESCV1, "force_igd_desc_v1"},
@@ -99,7 +103,7 @@ static const struct {
 };
 
 int
-readoptionsfile(const char * fname)
+readoptionsfile(const char * fname, int debug_flag)
 {
 	FILE *hfile = NULL;
 	char buffer[1024];
@@ -164,8 +168,9 @@ readoptionsfile(const char * fname)
 			tmp = realloc(upnppermlist, sizeof(struct upnpperm) * (num_upnpperm+1));
 			if(tmp == NULL)
 			{
-				fprintf(stderr, "memory allocation error. Permission line in file %s line %d\n",
+				INIT_PRINT_ERR("memory allocation error. Permission line in file %s line %d\n",
 				        fname, linenum);
+				return -1;
 			}
 			else
 			{
@@ -177,8 +182,9 @@ readoptionsfile(const char * fname)
 				}
 				else
 				{
-					fprintf(stderr, "parsing error file %s line %d : %s\n",
+					INIT_PRINT_ERR("parsing error file %s line %d : %s\n",
 					        fname, linenum, name);
+					return -1;
 				}
 			}
 			continue;
@@ -190,21 +196,23 @@ readoptionsfile(const char * fname)
 			tmp = realloc(dscp_values_list, sizeof(struct dscp_values) * (num_dscp_values+1));
 			if(tmp == NULL)
 			{
-				fprintf(stderr, "memory allocation error. DSCP line in file %s line %d\n",
+				INIT_PRINT_ERR("memory allocation error. DSCP line in file %s line %d\n",
 				        fname, linenum);
+				return -1;
 			}
 			else
 			{
 				dscp_values_list = tmp;
 				/* parse the rule */
-				if(read_learn_dscp_line(dscp_values_list + num_dscp_values, name) >= 0)
+				if(read_learn_dscp_line(dscp_values_list + num_dscp_values, name, debug_flag) >= 0)
 				{
 					num_dscp_values++;
 				}
 				else
 				{
-					fprintf(stderr, "parsing error file %s line %d : %s\n",
+					INIT_PRINT_ERR("parsing error file %s line %d : %s\n",
 					        fname, linenum, name);
+					return -1;
 				}
 			}
 			continue;
@@ -212,9 +220,9 @@ readoptionsfile(const char * fname)
 #endif /* PCP_SADSCP */
 		if(!(equals = strchr(name, '=')))
 		{
-			fprintf(stderr, "parsing error file %s line %d : %s\n",
+			INIT_PRINT_ERR("parsing error file %s line %d : %s\n",
 			        fname, linenum, name);
-			continue;
+			return -1;
 		}
 
 		/* remove ending whitespaces */
@@ -243,16 +251,18 @@ readoptionsfile(const char * fname)
 
 		if(id == UPNP_INVALID)
 		{
-			fprintf(stderr, "invalid option in file %s line %d : %s=%s\n",
+			INIT_PRINT_ERR("invalid option in file %s line %d : %s=%s\n",
 			        fname, linenum, name, value);
+			return -1;
 		}
 		else
 		{
 			tmp = realloc(ary_options, (num_options + 1) * sizeof(struct option));
 			if(tmp == NULL)
 			{
-				fprintf(stderr, "memory allocation error. Option in file %s line %d.\n",
+				INIT_PRINT_ERR("memory allocation error. Option in file %s line %d.\n",
 				        fname, linenum);
+				return -1;
 			}
 			else
 			{
@@ -261,8 +271,9 @@ readoptionsfile(const char * fname)
 				tmp = realloc(string_repo, string_repo_len + len);
 				if(tmp == NULL)
 				{
-					fprintf(stderr, "memory allocation error, Option value in file %s line %d : %s=%s\n",
+					INIT_PRINT_ERR("memory allocation error, Option value in file %s line %d : %s=%s\n",
 					        fname, linenum, name, value);
+					return -1;
 				}
 				else
 				{
