@@ -1,8 +1,8 @@
-/* $Id: upnpdescgen.c,v 1.88 2020/06/06 17:50:49 nanard Exp $ */
+/* $Id: upnpdescgen.c,v 1.89 2021/08/21 08:16:16 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * MiniUPnP project
  * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
- * (c) 2006-2020 Thomas Bernard
+ * (c) 2006-2021 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -887,11 +887,11 @@ strcat_int(char * str, int * len, int * tmplen, int i)
  * This way, the progam stack usage is kept low */
 static char *
 genXML(char * str, int * len, int * tmplen,
-                   const struct XMLElt * p)
+                   const struct XMLElt * p,
+       int force_igd1)
 {
 #define GENXML_STACK_SIZE 16
 	unsigned short i, j;
-	unsigned long k;
 	int top;
 	const char * eltname, *s;
 	char c;
@@ -928,7 +928,7 @@ genXML(char * str, int * len, int * tmplen,
 				str = strcat_str(str, len, tmplen, p[i].data);
 #ifdef IGD_V2
 				/* checking a single 'u' saves us 4 strcmp() calls most of the time */
-				if (GETFLAG(FORCEIGDDESCV1MASK) && (p[i].data[0] == 'u'))
+				if (force_igd1 && (p[i].data[0] == 'u'))
 				{
 					if ((strcmp(p[i].data, DEVICE_TYPE_IGD) == 0) ||
 						(strcmp(p[i].data, DEVICE_TYPE_WAN) == 0)  ||
@@ -943,6 +943,7 @@ genXML(char * str, int * len, int * tmplen,
 				str = strcat_str(str, len, tmplen, eltname);
 				str = strcat_char(str, len, tmplen, '>');
 			}
+unstack:
 			for(;;)
 			{
 				if(top < 0)
@@ -967,6 +968,15 @@ genXML(char * str, int * len, int * tmplen,
 		}
 		else
 		{
+			unsigned long k = (unsigned long)p[i].data;
+#ifdef IGD_V2
+			if((force_igd1 && (p[k & 0xffff].eltname[0] == '/')) &&
+			   (strcmp(p[k & 0xffff].data, "urn:schemas-upnp-org:service:DeviceProtection:1") == 0 ||
+			    strcmp(p[k & 0xffff].data, "urn:schemas-upnp-org:service:WANIPv6FirewallControl:1") == 0)) {
+				/* Skip the child element */
+				goto unstack;
+			}
+#endif
 			/* node with child(ren) */
 			/*printf("<%s>\n", eltname); */
 			str = strcat_char(str, len, tmplen, '<');
@@ -979,7 +989,6 @@ genXML(char * str, int * len, int * tmplen,
 				str = strcat_str(str, len, tmplen, configid_str);
 			}
 			str = strcat_char(str, len, tmplen, '>');
-			k = (unsigned long)p[i].data;
 			i = k & 0xffff;
 			j = i + (k >> 16);
 			if(top < (GENXML_STACK_SIZE - 1)) {
@@ -1003,7 +1012,7 @@ genXML(char * str, int * len, int * tmplen,
  *   the returned string.
  * - tmp_uuid argument is used to build the uuid string */
 char *
-genRootDesc(int * len)
+genRootDesc(int * len, int force_igd1)
 {
 	char * str;
 	int tmplen;
@@ -1014,7 +1023,7 @@ genRootDesc(int * len)
 	* len = strlen(xmlver);
 	/*strcpy(str, xmlver); */
 	memcpy(str, xmlver, *len + 1);
-	str = genXML(str, len, &tmplen, rootDesc);
+	str = genXML(str, len, &tmplen, rootDesc, force_igd1);
 	str[*len] = '\0';
 	return str;
 }
@@ -1023,7 +1032,7 @@ genRootDesc(int * len)
  * Generate service description with allowed methods and
  * related variables. */
 static char *
-genServiceDesc(int * len, const struct serviceDesc * s)
+genServiceDesc(int * len, const struct serviceDesc * s, int force_igd1)
 {
 	int i, j;
 	const struct action * acts;
@@ -1054,6 +1063,12 @@ genServiceDesc(int * len, const struct serviceDesc * s)
 	str = strcat_str(str, len, &tmplen, "<actionList>");
 	while(acts[i].name)
 	{
+#if defined(IGD_V2)
+		/* fake a IGD v1 for Microsoft clients :
+		 * no DeletePortMappingRange, GetListOfPortMappings, AddAnyPortMapping */
+		if (force_igd1 && strcmp(acts[i].name, "DeletePortMappingRange") == 0)
+			break;
+#endif
 		str = strcat_str(str, len, &tmplen, "<action><name>");
 		str = strcat_str(str, len, &tmplen, acts[i].name);
 		str = strcat_str(str, len, &tmplen, "</name>");
@@ -1181,40 +1196,40 @@ genServiceDesc(int * len, const struct serviceDesc * s)
 /* genWANIPCn() :
  * Generate the WANIPConnection xml description */
 char *
-genWANIPCn(int * len)
+genWANIPCn(int * len, int force_igd1)
 {
-	return genServiceDesc(len, &scpdWANIPCn);
+	return genServiceDesc(len, &scpdWANIPCn, force_igd1);
 }
 
 /* genWANCfg() :
  * Generate the WANInterfaceConfig xml description. */
 char *
-genWANCfg(int * len)
+genWANCfg(int * len, int force_igd1)
 {
-	return genServiceDesc(len, &scpdWANCfg);
+	return genServiceDesc(len, &scpdWANCfg, force_igd1);
 }
 
 #ifdef ENABLE_L3F_SERVICE
 char *
-genL3F(int * len)
+genL3F(int * len, int force_igd1)
 {
-	return genServiceDesc(len, &scpdL3F);
+	return genServiceDesc(len, &scpdL3F, force_igd1);
 }
 #endif
 
 #ifdef ENABLE_6FC_SERVICE
 char *
-gen6FC(int * len)
+gen6FC(int * len, int force_igd1)
 {
-	return genServiceDesc(len, &scpd6FC);
+	return genServiceDesc(len, &scpd6FC, force_igd1);
 }
 #endif
 
 #ifdef ENABLE_DP_SERVICE
 char *
-genDP(int * len)
+genDP(int * len, int force_igd1)
 {
-	return genServiceDesc(len, &scpdDP);
+	return genServiceDesc(len, &scpdDP, force_igd1);
 }
 #endif
 
