@@ -1,9 +1,9 @@
-/* $Id: upnphttp.c,v 1.109 2020/11/11 12:20:59 nanard Exp $ */
+/* $Id: upnphttp.c,v 1.112 2021/08/21 08:14:18 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * Project :  miniupnp
  * Website :  http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
  * Author :   Thomas Bernard
- * Copyright (c) 2005-2020 Thomas Bernard
+ * Copyright (c) 2005-2021 Thomas Bernard
  * This software is subject to the conditions detailed in the
  * LICENCE file included in this distribution.
  * */
@@ -29,7 +29,7 @@
 #include "upnpsoap.h"
 #include "upnpevents.h"
 #include "upnputils.h"
-#ifdef RANDOMIZE_URLS
+#if defined(RANDOMIZE_URLS) || defined(DYNAMIC_OS_VERSION)
 #include "upnpglobalvars.h"
 #endif /* RANDOMIZE_URLS */
 
@@ -308,6 +308,11 @@ ParseHttpHeaders(struct upnphttp * h)
 					syslog(LOG_DEBUG, "\"Expect: 100-Continue\" header detected");
 				}
 			}
+			else if(strncasecmp(line, "user-agent:", 11) == 0)
+			{
+				if(strcasestr(line + 11, "microsoft") != NULL)
+					h->respflags |= FLAG_MS_CLIENT;
+			}
 #ifdef ENABLE_EVENTS
 			else if(strncasecmp(line, "Callback:", 9)==0)
 			{
@@ -458,11 +463,19 @@ sendDummyDesc(struct upnphttp * h)
 
 /* Sends the description generated by the parameter */
 static void
-sendXMLdesc(struct upnphttp * h, char * (f)(int *))
+sendXMLdesc(struct upnphttp * h, char * (f)(int *, int))
 {
 	char * desc;
 	int len;
-	desc = f(&len);
+#ifdef IGD_V2
+#ifdef DEBUG
+	if(h->respflags & FLAG_MS_CLIENT)
+		syslog(LOG_DEBUG, "MS Client, forcing IGD v1");
+#endif /* DEBUG */
+	desc = f(&len, GETFLAG(FORCEIGDDESCV1MASK) || (h->respflags & FLAG_MS_CLIENT));
+#else
+	desc = f(&len, 0);
+#endif
 	if(!desc)
 	{
 		static const char error500[] = "<HTML><HEAD><TITLE>Error 500</TITLE>"
@@ -746,7 +759,7 @@ ProcessHttpQuery_upnphttp(struct upnphttp * h)
 {
 	static const struct {
 		const char * path;
-		char * (* f)(int *);
+		char * (* f)(int *, int);
 	} path_desc[] = {
 		{ ROOTDESC_PATH, genRootDesc},
 		{ WANIPC_PATH, genWANIPCn},
@@ -1103,12 +1116,11 @@ BuildHeader_upnphttp(struct upnphttp * h, int respcode,
                      const char * respmsg,
                      int bodylen)
 {
-	int templen;
+	int templen = sizeof(httpresphead) + 256 + bodylen;
 	if(!h->res_buf ||
-	   h->res_buf_alloclen < ((int)sizeof(httpresphead) + 256 + bodylen)) {
+	   h->res_buf_alloclen < templen) {
 		if(h->res_buf)
 			free(h->res_buf);
-		templen = sizeof(httpresphead) + 256 + bodylen;
 		h->res_buf = (char *)malloc(templen);
 		if(!h->res_buf) {
 			syslog(LOG_ERR, "malloc error in BuildHeader_upnphttp()");
@@ -1118,10 +1130,14 @@ BuildHeader_upnphttp(struct upnphttp * h, int respcode,
 	}
 	h->res_sent = 0;
 	h->res_buflen = snprintf(h->res_buf, h->res_buf_alloclen,
-	                         httpresphead, h->HttpVer,
+	                         httpresphead, h->HttpVer,	/* HTTP/x.x */
 	                         respcode, respmsg,
-	                         (h->respflags&FLAG_HTML)?"text/html":"text/xml; charset=\"utf-8\"",
-							 bodylen);
+	                         (h->respflags&FLAG_HTML)?"text/html":"text/xml; charset=\"utf-8\"",	/* Content-Type: */
+	                         bodylen		/* Content-Length: */
+#ifdef DYNAMIC_OS_VERSION
+	                         , os_version	/* Server: */
+#endif
+	                        );
 	/* Content-Type MUST be 'text/xml; charset="utf-8"' according to UDA v1.1 */
 	/* Content-Type MUST be 'text/xml' according to UDA v1.0 */
 	/* Additional headers */
@@ -1281,63 +1297,8 @@ SendResp_upnphttp(struct upnphttp * h)
 void
 SendRespAndClose_upnphttp(struct upnphttp * h)
 {
-	ssize_t n;
-
-	while (h->res_sent < h->res_buflen)
-	{
-#ifdef ENABLE_HTTPS
-		if(h->ssl) {
-			n = SSL_write(h->ssl, h->res_buf + h->res_sent,
-			         h->res_buflen - h->res_sent);
-		} else {
-			n = send(h->socket, h->res_buf + h->res_sent,
-			         h->res_buflen - h->res_sent, 0);
-		}
-#else
-		n = send(h->socket, h->res_buf + h->res_sent,
-		         h->res_buflen - h->res_sent, 0);
-#endif
-		if(n<0)
-		{
-#ifdef ENABLE_HTTPS
-			if(h->ssl) {
-				int err;
-				err = SSL_get_error(h->ssl, n);
-				if(err == SSL_ERROR_WANT_READ || err == SSL_ERROR_WANT_WRITE) {
-					/* try again later */
-					h->state = ESendingAndClosing;
-					return;
-				}
-				syslog(LOG_ERR, "SSL_write() failed");
-				syslogsslerr();
-				break; /* avoid infinite loop */
-			} else {
-#endif
-			if(errno == EINTR)
-				continue;	/* try again immediately */
-			if(errno == EAGAIN || errno == EWOULDBLOCK)
-			{
-				/* try again later */
-				h->state = ESendingAndClosing;
-				return;
-			}
-			syslog(LOG_ERR, "send(res_buf): %m");
-			break; /* avoid infinite loop */
-#ifdef ENABLE_HTTPS
-			}
-#endif
-		}
-		else if(n == 0)
-		{
-			syslog(LOG_ERR, "send(res_buf): %d bytes sent (out of %d)",
-							h->res_sent, h->res_buflen);
-			break;
-		}
-		else
-		{
-			h->res_sent += n;
-		}
-	}
-	CloseSocket_upnphttp(h);
+	if (SendResp_upnphttp(h))
+		CloseSocket_upnphttp(h);
+	else
+		h->state = ESendingAndClosing;
 }
-
