@@ -1,8 +1,8 @@
-/* $Id: upnpredirect.c,v 1.94 2018/05/03 08:27:40 nanard Exp $ */
+/* $Id: upnpredirect.c,v 1.98 2020/12/03 23:29:53 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * MiniUPnP project
  * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
- * (c) 2006-2018 Thomas Bernard
+ * (c) 2006-2020 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -356,6 +356,10 @@ upnp_redirect(const char * rhost, unsigned short eport,
 		                 "%hu->%s:%hu %s", eport, iaddr, iport, protocol);
 		return -3;
 	}
+
+	if (desc == NULL)
+		desc = "";	/* assume empty description */
+
 	/* IGDv1 (WANIPConnection:1 Service Template Version 1.01 / Nov 12, 2001)
 	 * - 2.2.20.PortMappingDescription :
 	 *  Overwriting Previous / Existing Port Mappings:
@@ -440,6 +444,8 @@ upnp_redirect_internal(const char * rhost, unsigned short eport,
 {
 	/*syslog(LOG_INFO, "redirecting port %hu to %s:%hu protocol %s for: %s",
 		eport, iaddr, iport, protocol, desc);			*/
+	if(disable_port_forwarding)
+		return -1;
 	if(add_redirect_rule2(ext_if_name, rhost, eport, iaddr, iport, proto,
 	                      desc, timestamp) < 0) {
 		return -1;
@@ -573,14 +579,16 @@ upnp_delete_redirection(unsigned short eport, const char * protocol)
 	return _upnp_delete_redir(eport, proto_atoi(protocol));
 }
 
-/* upnp_get_portmapping_number_of_entries()
- * TODO: improve this code. */
+/* upnp_get_portmapping_number_of_entries() */
 int
 upnp_get_portmapping_number_of_entries()
 {
+#if defined(USE_PF) || defined(USE_NFTABLES)
+	return get_redirect_rule_count(ext_if_name);
+#else
 	int n = 0, r = 0;
 	unsigned short eport, iport;
-	char protocol[4], iaddr[32], desc[64], rhost[32];
+	char protocol[8], iaddr[32], desc[64], rhost[32];
 	unsigned int leaseduration;
 	do {
 		protocol[0] = '\0'; iaddr[0] = '\0'; desc[0] = '\0';
@@ -592,6 +600,7 @@ upnp_get_portmapping_number_of_entries()
 		n++;
 	} while(r==0);
 	return (n-1);
+#endif
 }
 
 /* functions used to remove unused rules
@@ -779,4 +788,3 @@ write_ruleset_details(int s)
 	}
 }
 #endif
-
